<?php
namespace NicolasBejean\CategoryWidget\Block\Adminhtml\CategoryWidget;

use \Magento\Backend\Block\Template;
use \Magento\Backend\Block\Template\Context;
use \Magento\Framework\Exception\LocalizedException;
use \Magento\Framework\Exception\NoSuchEntityException;
use \Magento\Framework\Serialize\Serializer\Json as JsonSerializer;

use \NicolasBejean\MediaManager\Model\Image as ImageModel;
use \NicolasBejean\MediaManager\Model\ResourceModel\Image\Collection;
use \NicolasBejean\MediaManager\Model\ResourceModel\Image\CollectionFactory as ImageCollectionFactory;
use \NicolasBejean\MediaManager\Model\ImageRepository;

use \NicolasBejean\CategoryWidget\Model\CategoryWidget as CategoryWidgetModel;
use \NicolasBejean\CategoryWidget\Model\CategoryWidgetRepository;

use \Exception;

/**
 * Class AssignCategories
 *
 * @category PHP
 * @package  NicolasBejean\CategoryWidget\Block\Adminhtml\CategoryWidget
 * @author   Nicolas Béjean <nicolas@bejean.eu>
 * @license  https://github.com/nicolasbejean/category-widget/blob/master/licence.txt BSD Licence
 * @link     https://www.bejean.eu
 */
class AssignCategories extends Template
{
    /**
     * Block template
     *
     * @var string
     */
    protected $_template = 'NicolasBejean_CategoryWidget::categorywidget/edit/assign_categories.phtml';

    /**
     * @var JsonSerializer
     */
    private $jsonSerializer;

    /**
     * @var ImageCollectionFactory
     */
    private $imageCollectionFactory;

    /**
     * @var ImageRepository
     */
    private $imageRepository;

    /**
     * @var CategoryWidgetRepository
     */
    private $categoryWidgetRepository;

    /**
     * AssignImages constructor.
     *
     * @param Context $context
     * @param JsonSerializer $jsonSerializer
     * @param ImageCollectionFactory $imageCollectionFactory
     * @param ImageRepository $imageRepository
     * @param CategoryWidgetRepository $categoryWidgetRepository
     * @param array $data
     */
    public function __construct
    (
        Context                             $context,
        JsonSerializer                      $jsonSerializer,
        ImageCollectionFactory              $imageCollectionFactory,
        ImageRepository                     $imageRepository,
        CategoryWidgetRepository               $categoryWidgetRepository,
        array $data                         = []
    )
    {
        $this->jsonSerializer               = $jsonSerializer;
        $this->imageCollectionFactory       = $imageCollectionFactory;
        $this->imageRepository              = $imageRepository;
        $this->categoryWidgetRepository        = $categoryWidgetRepository;

        parent::__construct($context, $data);
    }

    /**
     * @return array
     */
    public function getImagesCollection()
    {
        try {
            /** @var Collection $collection */
            $collection = $this->imageCollectionFactory->create()->toOptionArray();

            if (!empty($collection)) {
                $result = array();

                foreach ($collection as $image) {
                    array_push($result, $this->getImage($image['value']));
                }

                return $result;
            }
        } catch (Exception $e) {
            return array($e->getMessage());
        }

        return array();
    }

    /**
     * @param $id
     * @return array
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    private function getImage (int $id)
    {
        /** @var ImageModel $image */
        $image = $this->imageRepository->getById($id);

        $result = array();
        $result['id'] = $image->getId();
        $result['path'] = $image->getPath();
        $result['alt'] = $image->getAlt();
        $result['is_active'] = $image->isActive();

        return $result;
    }

    /**
     * @param int $sliderId
     * @return array
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function getSlider (int $sliderId)
    {
        /** @var CategoryWidgetModel $categoryWidget */
        $categoryWidget = $this->categoryWidgetRepository->getById($sliderId);

        $array = explode('|', $categoryWidget->getContent());
        $array = array_map('intval', $array);

        $result = array();

        /** @var array $image */
        foreach ($array as $image) {
            array_push($result, $this->getImage($image));
        }

        return $result;
    }
}
