<?php
namespace NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget;

use \Magento\Backend\App\Action;
use \Magento\Backend\App\Action\Context;
use \Magento\Framework\Controller\Result\Json;
use \Magento\Framework\Controller\ResultInterface;
use \NicolasBejean\CategoryWidget\Api\CategoryWidgetRepositoryInterface as CategoryWidgetRepository;
use \Magento\Framework\Controller\Result\JsonFactory;
use \NicolasBejean\CategoryWidget\Api\Data\CategoryWidgetInterface;
use \NicolasBejean\CategoryWidget\Model\CategoryWidget;
use \Exception;

/**
 * Class InlineEdit
 *
 * @category PHP
 * @package  NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget
 * @author   Nicolas Béjean <nicolas@bejean.eu>
 * @license  https://github.com/nicolasbejean/category-widget/blob/master/licence.txt BSD Licence
 * @link     https://www.bejean.eu
 */
class InlineEdit extends Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'NicolasBejean_CategoryWidget::global';

    /**
     * @var CategoryWidgetRepository
     */
    protected $categoryWidgetRepository;

    /**
     * @var JsonFactory
     */
    protected $jsonFactory;

    /**
     * @param Context                   $context
     * @param CategoryWidgetRepository     $categoryWidgetRepository
     * @param JsonFactory               $jsonFactory
     */
    public function __construct(
        Context                         $context,
        CategoryWidgetRepository           $categoryWidgetRepository,
        JsonFactory                     $jsonFactory
    ) {
        $this->categoryWidgetRepository    = $categoryWidgetRepository;
        $this->jsonFactory              = $jsonFactory;

        parent::__construct($context);
    }

    /**
     * @return ResultInterface
     */
    public function execute()
    {
        /** @var Json $resultJson */
        $resultJson = $this->jsonFactory->create();
        $error = false;
        $messages = [];

        if ($this->getRequest()->getParam('isAjax')) {
            $postItems = $this->getRequest()->getParam('items', []);
            if (!count($postItems)) {
                $messages[] = __('Please correct the data sent.');
                $error = true;
            } else {
                foreach (array_keys($postItems) as $id) {
                    /** @var CategoryWidget $categoryWidget */
                    $categoryWidget = $this->categoryWidgetRepository->getById($id);
                    try {
                        $categoryWidget->setData(array_merge($categoryWidget->getData(), $postItems[$id]));
                        $this->categoryWidgetRepository->save($categoryWidget);
                    } catch (Exception $e) {
                        $messages[] = $this->getErrorWithImageId(
                            $categoryWidget,
                            __($e->getMessage())
                        );
                        $error = true;
                    }
                }
            }
        }

        return $resultJson->setData([
            'messages' => $messages,
            'error' => $error
        ]);
    }

    /**
     * Add Category Widget title to error message
     *
     * @param CategoryWidgetInterface $categoryWidget
     * @param string $errorText
     * @return string
     */
    protected function getErrorWithImageId(CategoryWidgetInterface $categoryWidget, $errorText)
    {
        return '[Category Widget ID: ' . $categoryWidget->getId() . '] ' . $errorText;
    }
}
